#!/usr/bin/env python3
#
# Written by Axel Hutt, INRIA Nancy Grand Est, France
# May 2022
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import matplotlib.pyplot as plt
from scipy.fft import fft, fftfreq
import numpy as np

class signal_class:
	def __init__(self):
		self.T  = 10.0 # duration of signal
		self.f1 = 0.5 # frequency
		self.f2 = 1.0 # frequency
		self.dt = 0.1
		self.num_time = int(self.T/self.dt) 

	def func(self,f,t):
		return np.cos(2*np.pi*f*t)

	def define_signal(self):
		self.signal = np.zeros(self.num_time)
		self.time   = np.zeros(self.num_time)
		for i in range(self.num_time):
			t = float(i)*self.dt
			self.signal[i] = self.func(self.f1,t)+2.0*self.func(self.f2,t)
			self.time[i] = t
			
	def compute_DFT(self):
		DFT = fft(self.signal)
		self.DFT_of_signal = DFT[:self.num_time//2]
		self.freqs = fftfreq(self.num_time, self.dt)[:self.num_time//2]
	
	def plot(self):
		fign=plt.figure(1)
		
		ax = fign.add_subplot(211)
		# in indices: 
		#plt.plot(range(self.num_time),self.signal,'k')
		plt.plot(self.time,self.signal,'k')
		plt.xlabel("time [s]")
		plt.ylabel("signal")
		
		ax = fign.add_subplot(212)
		# in Fourier mode numbers:
		#plt.plot(range(self.num_time//2),np.abs(self.DFT_of_signal),'k')
		plt.plot(self.freqs,np.abs(self.DFT_of_signal),'k')
		plt.xlabel("frequency [Hz]")
		plt.ylabel("|DFT|")
		
		fign.subplots_adjust(hspace=0.5)
		plt.show()

S = signal_class()
S.define_signal()
S.compute_DFT()
S.plot()